select count(*) from gnats where category ="gcc" and arrival < "91-05-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "91-06-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "91-07-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "91-08-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "91-09-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "91-10-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "91-11-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "91-12-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-01-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-02-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-03-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-04-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-05-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-06-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-07-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-08-01 00:00";
select count(*) from gnats where category ="gcc" and arrival < "92-09-01 00:00";
